@@ -18,12 +18,15 @@
 from __future__ import absolute_import
 
 import os
+import shutil
 import sys
 
 import gobject
 import gtk
 
+from solfege import abstract
 from solfege import cfg
+from solfege import filesystem
 from solfege import gu
 from solfege import languages
 from solfege import mpd
@@ -395,6 +398,32 @@ class ConfigWindow(gtk.Dialog, cfg.ConfigUtils):
             self.create_win32_sound_page()
         else:
             self.create_linux_sound_page()
+        ##############
+        # statistics #
+        ##############
+        page_vbox = gu.hig_dlg_vbox()
+        self.g_notebook.append_page(page_vbox, gtk.Label(_("Statistics")))
+        box, category_vbox = gu.hig_category_vbox(_("Statistics from 3.15 and older"))
+        page_vbox.pack_start(box, False)
+        self.g_old_stat_info = gtk.Label()
+        self.g_old_stat_info.set_line_wrap(True)
+        self.g_old_stat_info.set_alignment(0.0, 0.5)
+        category_vbox.pack_start(self.g_old_stat_info, False)
+        #
+        self.g_delete_old_statistics = gtk.Button(stock=gtk.STOCK_DELETE)
+        self.g_delete_old_statistics.connect('clicked', self.delete_obsolete_statistics)
+        category_vbox.pack_start(self.g_delete_old_statistics, False)
+        box, category_vbox = gu.hig_category_vbox(_("Statistics"))
+        page_vbox.pack_start(box, False)
+        self.g_stat_info = gtk.Label()
+        self.g_stat_info.set_line_wrap(True)
+        self.g_stat_info.set_alignment(0.0, 0.5)
+        category_vbox.pack_start(self.g_stat_info, False)
+        b = gtk.Button(stock=gtk.STOCK_DELETE)
+        b.connect('clicked', self.delete_statistics)
+        category_vbox.pack_start(b, False)
+        self.update_statistics_info()
+        self.update_old_statistics_info()
         self.show_all()
     def add_gui_for_external_programs(self, page_vbox):
         box, category_vbox = gu.hig_category_vbox(_("Audio File Players"))
@@ -457,7 +486,7 @@ class ConfigWindow(gtk.Dialog, cfg.ConfigUtils):
             cmd_combo = gtk.ComboBoxEntry(cmd_liststore)
             cmd_combo.set_tooltip_text(_("Enter the name of the program. An absolute path is required only if the executable is not found on the PATH."))
             cmd_combo.opts = gtk.combo_box_entry_new_text()
-            cmd_combo.opts.set_tooltip_text(_("The command line options required. Write %%s where you want the name of the file to be played. Or omit it to have it added to the end of the string."))
+            cmd_combo.opts.set_tooltip_text(_("The command line options required. Write %s where you want the name of the file to be played. Or omit it to have it added to the end of the string."))
             cmd_combo.child.set_text(cfg.get_string('sound/%s_player' % formatid))
             cmd_combo.opts.child.set_text(cfg.get_string('sound/%s_player_options' % formatid))
             for s in format['players']:
@@ -670,3 +699,41 @@ class ConfigWindow(gtk.Dialog, cfg.ConfigUtils):
                 self.g_synth_num.set_value(soundcard.synth.m_devnum)
             else:
                 self.g_synth.set_active(soundcard.synth.m_devnum + 1)
+    def delete_statistics(self, *w):
+        if gu.dialog_delete(_("Delete statistics and test results?"), self,
+                _("This will delete and recreate the file «%s».") % solfege.db.get_statistics_filename()):
+            restart = False
+            # We need to test for this, because get_view() can also return the front page
+            if isinstance(solfege.win.get_view(), abstract.Gui):
+                solfege.win.get_view().on_end_practise()
+                restart = True
+            solfege.db.reset_database()
+            if restart:
+                solfege.win.get_view().on_start_practise()
+                if solfege.win.get_view().m_t.m_P and solfege.win.get_view().m_t.m_statistics:
+                    solfege.db.validate_stored_statistics(solfege.win.get_view().m_t.m_P.m_filename)
+        self.update_statistics_info()
+    def delete_obsolete_statistics(self, *w):
+        if gu.dialog_delete(_("Delete obsolete statistics?"), self,
+                _("This will delete the directory «%s».") % os.path.join(filesystem.app_data(), u"statistics")):
+            try:
+                shutil.rmtree(os.path.join(filesystem.app_data(), u"statistics"))   
+            except OSError, e:
+                gu.display_exception_message(e)
+            self.update_old_statistics_info()
+    def update_old_statistics_info(self):
+        path = os.path.join(filesystem.app_data(), u'statistics')
+        if os.path.exists(path):
+            count = 1 # count app_data()/statistics too
+        else:
+            count = 0
+        for dirpath, dirnames, filenames in os.walk(path):
+            count += len(dirnames) + len(filenames)
+        if count:
+            self.g_old_stat_info.set_text(_("You have %i files and directories storing statistics in the old format that only Solfege 3.14 and older will use. These files are not useful any more unless you want to downgrade to Solfege 3.14.") % count)
+        else:
+            self.g_old_stat_info.set_text(_("No obsolete statistics found."))
+        self.g_delete_old_statistics.set_sensitive(bool(count))
+    def update_statistics_info(self):
+        self.g_stat_info.set_text(_("Statistics of %(practise_count)i practise sessions and %(test_count)i tests from %(exercises)i different exercises.") % solfege.db.get_statistics_info())
+
